/*
 * Copyright 2010 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.unitesk.requality.eclipse.editors.panels;

import java.util.List;

import org.eclipse.swt.SWT;
import org.eclipse.swt.browser.Browser;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Text;
import org.eclipse.ui.IWorkbenchPage;
import org.eclipse.ui.PartInitException;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.forms.events.HyperlinkEvent;
import org.eclipse.ui.forms.events.IHyperlinkListener;
import org.eclipse.ui.forms.widgets.Hyperlink;

import com.unitesk.requality.eclipse.editors.NodeEditorInput;
import com.unitesk.requality.eclipse.editors.NodeIdVerifyListener;
import com.unitesk.requality.eclipse.editors.browser.ReqMarker;
import com.unitesk.requality.nodetypes.Document;
import com.unitesk.requality.nodetypes.Location;
import com.unitesk.requality.nodetypes.Requirement;

public class RequirementMainPropertiesPanel extends AbstractPanel {
    private static final String EDIT_DESC = "Description:";
    private static final String EDIT_UID = "Requirement identifier:";
    private static final String EDIT_ALT = "Alternative description:";
    private static final String BUT_LOOK = "Look...";
    private static final String EDIT_NAME = "Name:";

    private static int HEIGHT_MULT = 100;

    private Requirement req;

    private Text IdText;
    private Text nameText;
    private Text altText;

    public String getId() {
        return IdText.getText();
    }

    public String getDescription() {
        return altText.getText();
    }

    public String getReqName() {
        if (nameLock.getSelection()) {
            return nameText.getText();
        } else {
            return getId();
        }
    }

    public void setRequirement(Requirement req) {
        this.req = req;
        if (isDisposed()) {
            return;
        }
        IdText.setText(req.getId());
        IdText.addVerifyListener(new NodeIdVerifyListener(req.getParent()));
        altText.setText(req.getDescription());
        nameText.setText(req.getName());
        createLocations(req.getLocations());

        if (req.getName().equals(req.getId())) {
            nameLock.setSelection(false);
            nameText.setEnabled(false);
        } else {
            nameLock.setSelection(true);
            nameText.setEnabled(true);
        }
    }

    public RequirementMainPropertiesPanel(Composite parent, int style) {
        super(parent, style | SWT.NONE);
        setLayout(new GridLayout(1, false));

        new Label(this, SWT.NONE).setText(EDIT_UID);
        IdText = new Text(this, SWT.BORDER);
        IdText.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));

        nameLock = new Button(this, SWT.CHECK);
        nameLock.setText(EDIT_NAME);
        nameLock.setSelection(false);
        nameLock.addSelectionListener(new SelectionListener() {
            @Override
            public void widgetSelected(SelectionEvent e) {
                nameText.setEnabled(nameLock.getSelection());

            }

            @Override
            public void widgetDefaultSelected(SelectionEvent e) {
                widgetSelected(e);
            }
        });

        nameText = new Text(this, SWT.BORDER);
        nameText.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));

        new Label(this, SWT.NONE).setText(EDIT_ALT);
        altText = new Text(this, SWT.BORDER | SWT.H_SCROLL | SWT.V_SCROLL
                | SWT.WRAP);

        GridData altTextLayoutData = new GridData(GridData.FILL_HORIZONTAL);
        altTextLayoutData.heightHint = HEIGHT_MULT;
        altTextLayoutData.widthHint = 0;
        altText.setLayoutData(altTextLayoutData);

        initModifyListener();
    }

    public void createLocations(List<Location> locations) {
        if (isDisposed())
            return;
        if (locationPanel == null) {
            locationPanel = new Composite(this, SWT.NONE);
            locationPanel.setLayout(new GridLayout(1, false));
            GridData data = new GridData(GridData.FILL_BOTH);
            data.minimumHeight=400;
            locationPanel.setLayoutData(data);
        }
        for (Control kid : locationPanel.getChildren()) {
            kid.dispose();
        }

        int x = getParent().getSize().x;
        if (locations.size() > 0) {
            Label l = new Label(locationPanel, SWT.NONE);
            l.setText(EDIT_DESC);
            Composite locContainer = new Composite(locationPanel, SWT.NONE);
            locContainer.setLayout(new GridLayout(2, false));
            locContainer.setLayoutData(new GridData(GridData.FILL_BOTH));
            for (Location loc : locations) {
                if (loc == null) {
                    // FIXME: check for location
                    continue;
                }
                String descr = loc.getDescription();
                String htmlPath = ((Document) loc.getParent()).getHTMLPath();
                if (descr.length() > 0) {
                    Browser descText = new Browser(locContainer, SWT.WRAP
                            | SWT.MULTI | SWT.BORDER | SWT.MOZILLA);
                    GridData ldata = new GridData(GridData.FILL_HORIZONTAL);
                    ldata.minimumHeight=400;
                    descText.setLayoutData(ldata);
                    
                    String prefix = "<html><head><meta http-equiv='Content-Type' content='text/html; charset=utf-8'/><base href='"
                            + "file:///" + htmlPath + "'/></head><body>";
                    String postfix = "</body></html>";

                    descText.setText(prefix + loc.getDescription() + postfix);
                    GridData locationTextLayoutData = new GridData(
                            GridData.FILL_BOTH);
                    descText.setLayoutData(locationTextLayoutData);
                    Hyperlink descrLink = new Hyperlink(locContainer, SWT.NONE);
                    descrLink.setText(BUT_LOOK);
                    descrLink.setHref(loc);
                    descrLink.addHyperlinkListener(openDoc);
                }
            }
        }
        getParent().pack();
        getParent().setSize(getParent().computeSize(x, SWT.DEFAULT));
    }

    public boolean isChanged() {
        if (req == null)
            return false; // No requirement can't be changed, isn't it?
        if (!getId().equals(req.getId()))
            return true;
        if (!getDescription().equals(req.getDescription()))
            return true;
        if (!this.getReqName().equals(req.getName()))
            return true;
        return false;
    }

    @Override
    protected void initModifyListener() {
        IdText.addListener(SWT.Modify, modifyListener);
        altText.addListener(SWT.Modify, modifyListener);
        nameText.addListener(SWT.Modify, modifyListener);
        nameLock.addListener(SWT.Selection, modifyListener);
    }

    @Override
    public boolean setFocus() {
        return IdText.setFocus();
    }

    public IHyperlinkListener openDoc = new IHyperlinkListener() {

        @Override
        public void linkExited(HyperlinkEvent e) {
            // TODO Auto-generated method stub

        }

        @Override
        public void linkEntered(HyperlinkEvent e) {
            // TODO Auto-generated method stub

        }

        @Override
        public void linkActivated(HyperlinkEvent link) {
            Object href = ((Hyperlink) link.widget).getHref();
            if (href instanceof Location) {
                Location loc = (Location) href;
                IWorkbenchPage page = PlatformUI.getWorkbench()
                        .getActiveWorkbenchWindow().getActivePage();
                if ((page != null) && (loc != null)) {
                    try {
                        String docId = loc.getParent().getQualifiedId();

                        Document doc = (Document) RequirementMainPropertiesPanel.this.req
                                .getTreeDB().getNode(docId);
                        NodeEditorInput input = new NodeEditorInput(doc);
                        input.setData(loc);
                        page.openEditor(input, ReqMarker.ID, true);
                    } catch (PartInitException e) {
                        e.printStackTrace();
                    }
                }
            }
        }
    };
    private Button nameLock;
    private Composite locationPanel;

    public void updateLocations(Requirement node) {
        createLocations(node.getLocations());
    }
}
