/*
 * Copyright 2009 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package com.unitesk.requality.eclipse.views.actions.requirements;

import org.eclipse.jface.viewers.ISelectionProvider;
import org.eclipse.ui.ISharedImages;
import org.eclipse.ui.IWorkbenchPage;
import org.eclipse.ui.PartInitException;
import org.eclipse.ui.PlatformUI;

import com.unitesk.requality.core.TreeNode;
import com.unitesk.requality.eclipse.Activator;
import com.unitesk.requality.eclipse.editors.NodeEditorInput;
import com.unitesk.requality.eclipse.editors.browser.AddSelectionToRequirement;
import com.unitesk.requality.eclipse.views.actions.CNFAction;
import com.unitesk.requality.nodetypes.Requirement;

public class AddRequirementAction extends CNFAction {
    private static final String ADD_REQUIREMENT = "Add Requirement...";

    @Override
    public boolean isEnabled() {
        TreeNode node = getSelectedNode();
        if (node == null) {
            return false;
        }
        if (!node.getType().equals("Requirement")) {
            return false;
        }
        return true;
    }
    public AddRequirementAction() {
    }

    public AddRequirementAction(ISelectionProvider provider) {
//        super(provider);
        setText(ADD_REQUIREMENT);
        setToolTipText(ADD_REQUIREMENT);
        setImageDescriptor(PlatformUI.getWorkbench().getSharedImages()
                .getImageDescriptor(ISharedImages.IMG_TOOL_NEW_WIZARD));
    }

    public void run() {
        TreeNode node = getSelectedNode();
        String id = AddSelectionToRequirement.getNextId(node);
        Requirement newReq = (Requirement) node.createChild(id, "Requirement");
        newReq.saveAttributes();
        try {
            IWorkbenchPage page = PlatformUI.getWorkbench()
                    .getActiveWorkbenchWindow().getActivePage();
            if (page == null)
                return;
            page.openEditor(new NodeEditorInput(newReq),
                    "com.unitesk.requality.editors.ReqEditor", true);
        } catch (PartInitException e) {
            e.printStackTrace();
        }
        Activator.getDefault().setActiveNode(newReq, "Requirement");
    }
}
