package com.unitesk.requality.eclipse.views.documents;

import java.io.IOException;
import java.io.Reader;
import java.io.Writer;
import java.util.Stack;
import java.util.UUID;

import javax.xml.stream.XMLEventFactory;
import javax.xml.stream.XMLEventReader;
import javax.xml.stream.XMLEventWriter;
import javax.xml.stream.XMLInputFactory;
import javax.xml.stream.XMLOutputFactory;
import javax.xml.stream.events.XMLEvent;

import com.unitesk.requality.core.TreeNode;
import com.unitesk.requality.nodetypes.Document;
import com.unitesk.requality.nodetypes.Location;
import com.unitesk.requality.nodetypes.Requirement;

public class HeadersDocumentProcessor extends DocumentProcessor
{
    @Override
    protected void process( String docId, Reader in, Writer out, TreeNode rootNode ) throws IOException
    {        
        try
        {
            XMLEventReader reader = XMLInputFactory.newInstance().createXMLEventReader( in );
            XMLEventWriter writer = XMLOutputFactory.newInstance().createXMLEventWriter( out );
            XMLEventFactory eventFactory = XMLEventFactory.newInstance();
            
            int headLevel = 0;
            int chIndex = 0;
    
            Stack<Requirement> rootNodes = new Stack<Requirement>();
            Stack<Integer> ids = new Stack<Integer>();
            rootNodes.push( (Requirement)rootNode );
            ids.push(0);
            Requirement req = null;
            Location location = null;
            String locationId = null;
            
            String name = null;
            
            while( reader.hasNext() )
            {
                XMLEvent event = (XMLEvent)reader.next();
                if( event.isStartElement() ) 
                {
                    String tagName = event.asStartElement().getName().getLocalPart();
                    int hl = headLevel( tagName );
                    if( hl > 0 ) 
                    {
                        headLevel = hl;
                        rootNodes.setSize( headLevel );
                        ids.setSize( headLevel + 1 );
                        Integer id = ids.get(headLevel);
                        ids.set( headLevel, (id!=null?id:0)+1 );
                        Requirement parent = null;
                        for( int i = rootNodes.size() - 1; i >= 0; i-- )
                        {
                            parent = rootNodes.get(i);
                            if( parent != null ) break;
                        }
                        req = (Requirement) parent.createChild( "" + ids.peek() );
                        rootNodes.push( req );
                        Document doc = (Document) req.getTreeDB().getNode(docId);
                        location = (Location) doc.createChild(UUID.randomUUID().toString(), "Location");
                        locationId = location.getId().toString();
                        req.addLocation( location.getQualifiedId() );
                        name = "";
                        chIndex = 0;
                    }
                    
                    writer.add(event);
                } else
                if( event.isEndElement() ) 
                {
                    String tagName = event.asEndElement().getName().getLocalPart();
                    int hl = headLevel( tagName );
                    if( hl > 0 ) 
                    {
                        headLevel = 0;
                        req.setName(name.trim());
                        req.saveAttributes();
//                        rootNode.getTreeDB().save(req);
                        req = null;
                        name = null;
                    }
                    writer.add(event);
                } else
                if( event.isCharacters() )
                {
                    String data = event.asCharacters().getData();
                    if( headLevel > 0 )
                    {
                        if( chIndex == 0 )
                        {
                            if( data.trim().length() > 0 ) chIndex++;
                        } else {
                            chIndex++;
                        }
                        if( chIndex > 0 )
                        {
                            name += data;
                            
                            writer.add( eventFactory.createStartElement("", "", "span" ) );
                            writer.add( eventFactory.createAttribute( "class", "requality_text id_" + locationId ) );
    
                            if( chIndex == 1 )
                            {
                                writer.add( eventFactory.createStartElement("", "", "a" ) );
                                writer.add( eventFactory.createAttribute( "name", locationId ) );
                                writer.add( eventFactory.createAttribute( "id", "id_" + locationId ) );
                                writer.add( eventFactory.createAttribute( "class", "requality_id" ) );
                                writer.add( eventFactory.createEndElement("", "", "a" ) );
                            }
                            
                            writer.add(event);
                            
                            writer.add( eventFactory.createEndElement("", "", "span" ) );
                        } else {
                            writer.add(event);
                        }
                        chIndex++;
                    } else {
                        writer.add(event);
                    }
                } else {
                    writer.add(event);
                }
            }
        } 
        catch( Throwable e)
        {
            throw new IOException( e );
        }
    }

    private int headLevel( String tagName )
    {
        if( tagName.length() != 2 ) return 0;
        char h = tagName.charAt(0);
        if( h != 'h' && h != 'H' ) return 0;
        int headLevel = tagName.charAt(1) - '0';
        if( headLevel < 1 || headLevel > 7 ) return 0;
        return headLevel;
    }
}
