package com.unitesk.requality.eclipse.views.documents;

import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.Reader;
import java.io.Writer;
import java.lang.reflect.InvocationTargetException;
import java.util.List;

import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.dialogs.ProgressMonitorDialog;
import org.eclipse.jface.operation.IRunnableWithProgress;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.ui.PlatformUI;

import com.unitesk.requality.core.TreeDB;
import com.unitesk.requality.core.TreeNode;
import com.unitesk.requality.eclipse.dialogs.SelectDocDialog;
import com.unitesk.requality.merge.MergeProcessor;
import com.unitesk.requality.nodetypes.Document;
import com.unitesk.requality.nodetypes.Location;
import com.unitesk.requality.nodetypes.Requirement;

public class LocationTracker extends DocumentProcessor {
    private Document oldDocument = null;
    private TreeDB db;

    @Override
    public boolean coufigure(Document doc) {
	this.db = doc.getTreeDB();
	Shell shell = PlatformUI.getWorkbench().getActiveWorkbenchWindow()
		.getShell();
	SelectDocDialog dialog = new SelectDocDialog(shell, db);
	boolean selectAgain;
	do {
	    selectAgain = false;
	    if (dialog.open() == SelectDocDialog.OK) {
		oldDocument = (Document) dialog.getResult();
		if (doc.getQualifiedId().equals(oldDocument.getQualifiedId())) {
		    MessageDialog
			    .openError(
				    shell,
				    "Same document",
				    "Same document selected. "
					    + "Select previous version of this document.");
		    selectAgain = true;
		} else {
		    return true;
		}
	    }
	} while (selectAgain);
	return false;
    }

    @Override
    protected void process(String docId, final Reader reader,
	    final Writer writer, TreeNode rootNode) throws IOException {
	Document newDoc = (Document) db.getNode(docId);
	final InputStreamReader oldIn = new InputStreamReader(
		new FileInputStream(oldDocument.getHTMLPath()), "UTF-8");
	final MergeProcessor mp = MergeProcessor.getInstance();
	try {
	    Shell shell = PlatformUI.getWorkbench().getActiveWorkbenchWindow()
		    .getShell();
	    ProgressMonitorDialog dialog = new ProgressMonitorDialog(shell);
	    dialog.run(true, false, new IRunnableWithProgress() {
		public void run(IProgressMonitor monitor) {
		    monitor.beginTask("Updating document...",
			    IProgressMonitor.UNKNOWN);
		    mp.merge(oldIn, reader, writer);
		    monitor.done();
		}
	    });
	} catch (InvocationTargetException e) {
	    e.printStackTrace();
	    MessageDialog.openInformation(PlatformUI.getWorkbench()
			.getActiveWorkbenchWindow().getShell(),
			"Document processing report", "Internal error occured");
	} catch (InterruptedException e) {
	    e.printStackTrace();
	    MessageDialog.openInformation(PlatformUI.getWorkbench()
			.getActiveWorkbenchWindow().getShell(),
			"Document processing report", "Internal error occured");
	}

	oldIn.close();

	List<String> persistentLocations = mp.getMergedIds();
	int processedLocations = 0;
	for (String id : persistentLocations) {
	    Location oldLoc = oldDocument.getLocation(id);
	    if (oldLoc != null) {
		Requirement[] reqs = oldLoc.getRequirements();
		Location loc = (Location) newDoc.createChild(id, "Location");
		loc.setDescription(oldLoc.getDescription());
		loc.saveAttributes();
		for (Requirement req : reqs) {
		    req.addLocation(loc.getQualifiedId());
		    req.saveAttributes();
		}
		processedLocations++;
	    }
	}
	MessageDialog.openInformation(PlatformUI.getWorkbench()
		.getActiveWorkbenchWindow().getShell(),
		"Document processing report", processedLocations
			+ " locations processed");
    }
}
