/*
 * Copyright 2009 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package com.unitesk.requality.eclipse.views.listeners;

import org.eclipse.jface.action.Action;
import org.eclipse.jface.viewers.ITreeSelection;
import org.eclipse.jface.viewers.TreeViewer;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.KeyEvent;
import org.eclipse.swt.events.KeyListener;

public class TreeViewerKeyListener implements KeyListener {
   
   private TreeViewer tree;
   
   private Action addReqAction;
   
   private Action removeReqAction;
   
   @SuppressWarnings( "unused" )
   private TreeViewerKeyListener() {
   }
   
   public TreeViewerKeyListener( TreeViewer tree, Action addReqAction, Action removeReqAction ) {
      this.tree= tree;
      this.addReqAction= addReqAction;
      this.removeReqAction= removeReqAction;
   }
   
   public void keyPressed( KeyEvent e ) {
      if ( e.keyCode == SWT.DEL ) {
         removeReqAction.run();
      }
      if ( e.keyCode == SWT.INSERT ) {
         addReqAction.run();
      }
      if ( e.keyCode == ' ' ) {
         ITreeSelection selected= ( ITreeSelection ) tree.getSelection();
         if ( selected.size() == 1 ) {
            Object data= selected.getFirstElement();
            if ( tree.getExpandedState( data ) == true )
               tree.collapseToLevel( data, 1 );
            else
               tree.expandToLevel( data, 1 );
         }
      }
   }
   
   public void keyReleased( KeyEvent e ) {
   }
   
}
