package com.unitesk.requality.nodetypes;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import java.util.UUID;

import com.unitesk.requality.core.Attribute;
import com.unitesk.requality.core.AttributeType;
import com.unitesk.requality.core.TreeDB;
import com.unitesk.requality.core.TreeNode;

public class Requirement extends TreeNode implements Comparable<TreeNode> {
    private static final String TYPE_NAME = "Requirement";
    private static final String ATTR_NAME = "_name";
    private static final String ATTR_LOCATIONS = "_locations";
    private static final String TYPE_ROOT_QID = "/Requirements";
    
    public static String getTypeRootQId() {
        return TYPE_ROOT_QID;
    }

    // Constructor for eclipse instancer
    public Requirement() {
        super(null, null);
    }

    public Requirement(TreeDB db, UUID uuid) {
        super(db, uuid);
        setType(TYPE_NAME);
    }

    @SuppressWarnings("unchecked")
    public List<String> getLocationQIds() {
        Object val = getAttributeValue(ATTR_LOCATIONS);
        if(val == null) {
            return Collections.EMPTY_LIST;
        }
        if (val instanceof List) {
            return (List<String>) getAttributeValue(ATTR_LOCATIONS);
        }
        throw new IllegalStateException(ATTR_LOCATIONS+" value must be string list");
    }

    public List<Location> getLocations() {
        List<Location> result = new ArrayList<Location>();
        for(String locQId: getLocationQIds()) {
            TreeNode locNode = getTreeDB().getNode(locQId);
            if( locNode != null && locNode instanceof Location )
            {
                result.add((Location)locNode);
            }
        }
        return Collections.unmodifiableList(result);
    }

    public Location getLocation(String QId) {
        if(getLocationQIds().contains(QId)) {
            return (Location)getTreeDB().getNode(QId);
        }
        return null;
    }

    @SuppressWarnings("unchecked")
    public void addLocation(String nodeQId) {
        List<String> locationNodes = (List<String>) getAttributeValue(ATTR_LOCATIONS);
        if (locationNodes == null ) {
            locationNodes = new ArrayList<String>();
        }
        locationNodes.add(nodeQId);
        putAttribute(new Attribute(AttributeType.LIST, ATTR_LOCATIONS,
                locationNodes));
    }

    @SuppressWarnings("unchecked")
    public boolean removeLocation(String nodeQId) {
        List<String> locations = (List<String>) getAttributeValue(ATTR_LOCATIONS);
        if (locations == null || !locations.remove(nodeQId)) {
            return false;
        }
        putAttribute(new Attribute(AttributeType.LIST, ATTR_LOCATIONS,
                locations));
        return true;
    }

    public void setName(String name) {
        putAttribute(new Attribute(AttributeType.RESOURCE, ATTR_NAME, name));
    }

    public String getName() {
        if (getAttributeValue(ATTR_NAME) == null)
            return getId();
        return (String) getAttributeValue(ATTR_NAME);
    }

    @Override
    public String toString() {
        return getId() + "(" + getUUId().toString() + ")";
    }

    @Override
    public int compareTo(TreeNode o) {
        if (o instanceof Requirement) {
            Requirement req = (Requirement) o;
            return getName().compareTo(req.getName());
        }
        else if(o instanceof TreeNode){
            TreeNode req = (TreeNode) o;
            return getName().compareTo(req.getId());
        }
        throw new IllegalArgumentException();
    }
}
