package com.unitesk.requality.tests.core;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertNotNull;
import static org.junit.Assert.assertNull;
import static org.junit.Assert.assertTrue;
import static org.junit.Assert.fail;

import java.util.Collection;

import org.junit.Before;
import org.junit.Ignore;
import org.junit.Test;

import com.unitesk.requality.core.MemoryStorage;
import com.unitesk.requality.core.TreeDB;
import com.unitesk.requality.core.TreeNode;
import com.unitesk.requality.core.TreeStorage;
import com.unitesk.requality.nodetypes.NodeType;

// TODO: in types: root folder for nodetype (first children of type will store in type folder)
// TODO: can node of this type be subroot

public class NodeTypeTest {
    private TreeDB db;
    private TreeStorage storage;

    @Before
    public void setUp() {
        storage = new MemoryStorage();
        db = new TreeDB(storage);
    }

    // TODO: test type creation with reserved type name
    @Test
    public void createType() {
        NodeType type =  new NodeType("TestType1", db);
        db.createType(type);
        assertEquals(4, db.getTypes().size());
        assertNotNull(db.getNodeType("TestType1"));
        // TODO: (Type Tests) type equals check
    }

    // TODO: test type creation with reserved type name
    @Test
    public void deleteType() {
        String newTypeName = "TestType1";
        NodeType type =  new NodeType(newTypeName, db);
        db.createType(type);
        assertEquals(4, db.getTypes().size());
        db.removeType(newTypeName);
        assertEquals(3, db.getTypes().size());
    }

    // DBRoot, NodeType and Node types are exists always
    @Test
    public void baseTypesCheck() {
        Collection<NodeType> types = db.getTypes().values();
        assertEquals("Must be only three base node types", 3, types.size());
        String[] names = new String[]{"DBRoot","NodeType","Node"};
        boolean found;
        for(String name : names) {
            found = false;
            for(NodeType t: types) {
                if(t.getName().equals(name)) {
                    found = true;
                    break;
                }
            }
            if(! found) {
                fail("'"+name+"' type not found");
            }
        }
    }

    @Test
    @Ignore
    public void createNewTypeNode() {
        NodeType newType = new NodeType("SimpleType", db);
        db.createType(newType);
    }
    
    @Test
    public void createSimpleTree() {
//        NodeType type = new NodeType("TypeForTest", db);


        TreeNode root = db.getRootNode();
        assertNotNull(root);
        assertEquals("/", root.getId());
        assertNull(root.getParent());
        assertEquals(db, root.getTreeDB());

        TreeNode hello = root.createChild("hello");
        assertNotNull(hello);
        assertEquals("hello", hello.getId());
        assertEquals(root, hello.getParent());

        assertEquals(1, root.sizeChildren());
        assertEquals(hello, root.getChild(0));

        TreeNode spec = root.createChild("spec", "NodeType");
        assertNotNull(spec);
        assertEquals(spec, root.findChild("spec"));
        assertEquals("NodeType", spec.getType());
    }

    // If user try to create a subroot with name of "types" TreeDB must throw exception
    @Test
    @Ignore
    public void typesBranchCreationProtection() {
       TreeNode root= db.getRootNode();
       boolean exceptionThrowed = false;
       try {
           root.createChild("Types");
       } catch (Exception e) {
           exceptionThrowed = true;
       }
       assertTrue(exceptionThrowed);
       TreeNode a = root.createChild("a");
       try {
           a.createChild("Types");
       } catch (Exception e) {
           fail();
       }
    }
}
