package com.unitesk.requality.tests.merge;

import java.util.HashMap;
import java.util.Map;

import org.w3c.dom.Document;

public class TestDataHelper {
    private static TestDataHelper instance = new TestDataHelper();
    private Map<String, Pair<Document, Document>> sanity;
    private Map<String, Pair<Document, Document>> debug;
    private Map<String, Pair<Document, Document>> acceptance;
    
    private TestDataHelper() {
	sanity = new HashMap<String, Pair<Document, Document>>();
	debug = new HashMap<String, Pair<Document, Document>>();
	acceptance = new HashMap<String, Pair<Document, Document>>();
	makeSanityData();
	makeDebugData();
	makeAcceptanceData();
    }

    private void makeSanityData() {
	Document ref = Utils.parseXml("./test-data/merge/sanity/sanity_marked.xml");
	Document upd = Utils.parseXml("./test-data/merge/sanity/sanity.xml");
	Pair<Document, Document> sanityDoc = new Pair<Document, Document>(ref, upd);
	sanity.put("Sanity", sanityDoc);
    }
    
    private void makeDebugData() {
	Document dbg_ref = Utils.parseXml("./test-data/merge/debug/debug_marked.xml");
	Document dbg_upd = Utils.parseXml("./test-data/merge/debug/debug.xml");
	Pair<Document, Document> dbgDoc = new Pair<Document, Document>(dbg_ref, dbg_upd);
	debug.put("Debug", dbgDoc);
	Document rfc_ref = Utils.parseXml("./test-data/merge/debug/rfc2821_marked.xhtml");
	Document rfc_upd = Utils.parseXml("./test-data/merge/debug/rfc2821.xhtml");
	Pair<Document, Document> rfcDoc = new Pair<Document, Document>(rfc_ref, rfc_upd);
	debug.put("Rfc", rfcDoc);
	Document wiki_ref = Utils.parseXml("./test-data/merge/debug/wikipedia_marked.xhtml");
	Document wiki_upd = Utils.parseXml("./test-data/merge/debug/wikipedia.xhtml");
	Pair<Document, Document> wikiDoc = new Pair<Document, Document>(wiki_ref, wiki_upd);
	debug.put("Wiki", wikiDoc);
	Document imap_ref = Utils.parseXml("./test-data/merge/debug/imap_marked.xhtml");
	Document imap_upd = Utils.parseXml("./test-data/merge/debug/imap.xhtml");
	Pair<Document, Document> imapDoc = new Pair<Document, Document>(imap_ref, imap_upd);
	debug.put("IMAP", imapDoc);
    }
    
    private void makeAcceptanceData() {
	Document rfc_ref = Utils.parseXml("./test-data/merge/acceptance/rfc_marked.xhtml");
	Document rfc_upd = Utils.parseXml("./test-data/merge/acceptance/rfc.xhtml");
	Pair<Document, Document> rfcDoc = new Pair<Document, Document>(rfc_ref, rfc_upd);
	acceptance.put("Rfc", rfcDoc);
	Document wiki_ref = Utils.parseXml("./test-data/merge/acceptance/wikipedia_marked.xhtml");
	Document wiki_upd = Utils.parseXml("./test-data/merge/acceptance/wikipedia.xhtml");
	Pair<Document, Document> wikiDoc = new Pair<Document, Document>(wiki_ref, wiki_upd);
	acceptance.put("Wiki", wikiDoc);
    }
    
    public Map<String, Pair<Document, Document>> getSanityData(){
	return this.sanity;
    }
    
    public Map<String, Pair<Document, Document>> getDebugData() {
	return this.debug;
    }
    
    public Map<String, Pair<Document, Document>> getAcceptanceData() {
	return this.acceptance;
    }
    
    public static TestDataHelper getInstance() {
	return instance;
    }
    
    public static void update() {
	instance = new TestDataHelper();
    }
}
