package com.unitesk.requality.tests.merge.acceptance;

import java.util.Map;

import junit.framework.TestCase;

import org.w3c.dom.Document;

import com.unitesk.requality.merge.MergeProcessor;
import com.unitesk.requality.tests.merge.TestDataHelper;
import com.unitesk.requality.tests.merge.Pair;

public class MergeAcceptanceTests extends TestCase{

    private MergeProcessor mp;
    private Map<String, Pair<Document, Document>> testData;
    
    public void setUp() {
	mp = MergeProcessor.getInstance();
	testData = TestDataHelper.getInstance().getAcceptanceData();
    }

    public void testWikipedia() {
	Document v1 = testData.get("Wiki").getKey();
	Document v2 = testData.get("Wiki").getValue();
	mp.merge(v1, v2);
	
	assertTrue(mp.getMergedIds().contains("ff19d8a2-1be5-4802-90c5-a746ca0bbf21"));
	assertFalse(mp.getMergedIds().contains("7813214b-8ee2-4587-b295-c2d09f92e237"));
	assertFalse(mp.getMergedIds().contains("a4741ae6-5b03-42f7-a828-d797cf82e0a4"));
	
	assertFalse(mp.getMissingIds().contains("ff19d8a2-1be5-4802-90c5-a746ca0bbf21"));
	assertTrue(mp.getMissingIds().contains("7813214b-8ee2-4587-b295-c2d09f92e237"));
	assertTrue(mp.getMissingIds().contains("a4741ae6-5b03-42f7-a828-d797cf82e0a4"));
	
	assertEquals(3, mp.getMissingIds().size());
	assertEquals(1, mp.getMergedIds().size());
	
	//FIXME implement functionality to merge this location
	assertTrue(!mp.getMergedIds().contains("7a7da054-40db-4331-bbb3-76e7dd7c6c3f"));
	assertFalse(!mp.getMissingIds().contains("7a7da054-40db-4331-bbb3-76e7dd7c6c3f"));
    }
    
    public void testRfc() {
	Document v1 = testData.get("Rfc").getKey();
	Document v2 = testData.get("Rfc").getValue();
	mp.merge(v1, v2);
	
	assertTrue(mp.getMergedIds().contains("9690a24d-987f-41b9-af64-ea8796d713b8"));
	assertTrue(mp.getMergedIds().contains("cb40419d-bbe6-433b-8d17-947eb247d0dd"));
	assertFalse(mp.getMergedIds().contains("e28f4155-657e-4f2a-ad9a-6ecb8fed4ab7"));
	assertFalse(mp.getMergedIds().contains("faed5308-a65a-42a3-bb90-fdd9d83cb883"));
	
	assertFalse(mp.getMissingIds().contains("9690a24d-987f-41b9-af64-ea8796d713b8"));
	assertFalse(mp.getMissingIds().contains("cb40419d-bbe6-433b-8d17-947eb247d0dd"));
	assertTrue(mp.getMissingIds().contains("e28f4155-657e-4f2a-ad9a-6ecb8fed4ab7"));
	assertTrue(mp.getMissingIds().contains("faed5308-a65a-42a3-bb90-fdd9d83cb883"));
	
	assertEquals(2, mp.getMissingIds().size());
	assertEquals(2, mp.getMergedIds().size());
    }
}
