package com.unitesk.requality.tests.merge.acceptance;

import java.io.File;
import java.lang.reflect.InvocationTargetException;
import java.util.ArrayList;
import java.util.List;

import junit.framework.TestCase;

import org.w3c.dom.Document;

import com.unitesk.requality.merge.Location;
import com.unitesk.requality.merge.MergeProcessor;
import com.unitesk.requality.merge.Tree;
import com.unitesk.requality.tests.merge.MergeProcessorAccessor;
import com.unitesk.requality.tests.merge.Pair;
import com.unitesk.requality.tests.merge.Utils;

public class PosixTest extends TestCase {

    private static final String markedPosixDir = "/tmp/legacyConv/";
    private static final String unmarkedPosixDir = "/tmp/posixConv/";
    private static final String mergedDir = "/tmp/merge/";
    private List<Pair<File, File>> testData;

    public void setUp() {
	testData = new ArrayList<Pair<File, File>>();
	prepareData();
	if (testData.size() == 0)
	    fail("No test data for tests");
	mergeData();
	System.out.println("Merge complete. Verifying...");
    }

    private void prepareData() {
	File sourceDir = new File(markedPosixDir);
	File targetDir = new File(unmarkedPosixDir);
	File[] sourceFiles = sourceDir.listFiles();
	File[] targetFiles = targetDir.listFiles();
	if (sourceFiles != null && targetFiles != null)
	    for (File src : sourceFiles) {
		for (File trg : targetFiles) {
		    //---------------------------------
		    if(!src.getName().contains("atan2"))
			continue;
		    //---------------------------------
		    if (src.getName().equals(trg.getName())) {
			Pair<File, File> docs = new Pair<File, File>(src, trg);
			testData.add(docs);
			break;
		    }
		}
	    }
    }

    private void mergeData() {
	int counter = 0;
	for (Pair<File, File> docs : testData) {
	    counter++;
	    Document src = Utils.parseXml(docs.getKey().getPath());
	    Document trg = Utils.parseXml(docs.getValue().getPath());
	    MergeProcessor.getInstance().merge(src, trg);
	    Utils.writeXmlFile(trg, mergedDir + docs.getKey().getName());
	    System.out.println("Merged " + mergedDir
		    + docs.getValue().getName() + " (" + counter + " of "
		    + testData.size() + ")");
	}
    }

    public void testPosixMerge() {
	int fail_count = 0;
	int total_locations = 0;
	int total_locations_merged = 0;
	int counter = 0;
	for (Pair<File, File> docs : testData) {
	    counter++;
	    Document source = Utils.parseXml(docs.getKey().getPath());
	    Document mergeResult = Utils.parseXml(mergedDir
		    + docs.getKey().getName());
	    System.out.println("Verifying " + docs.getKey().getPath()
		    + " versus " + mergedDir
		    + docs.getKey().getName() + " (" + counter + " of "
		    + testData.size() + ")");
	    MergeProcessorAccessor mpa = new MergeProcessorAccessor(
		    MergeProcessor.getInstance());
	    try {
		List<Location> locationsRef = mpa.lookupLocations(new Tree(
			source));
		List<Location> locationsMerged = mpa.lookupLocations(new Tree(
			mergeResult));
		total_locations += locationsRef.size();
		total_locations_merged += locationsMerged.size();
		List<Location> successfullyMerged = new ArrayList<Location>();
		for (Location locRef : locationsRef) {
		    String idRef = locRef.getLocationId();
		    for (Location locMerged : locationsMerged) {
			if (locMerged.getLocationId().equals(idRef)
				&& !successfullyMerged.contains(locMerged)) {
			    if (!locRef.getLocationText().equals(
				    locMerged.getLocationText())) {
				fail_count++;
				System.out.println("Failed to merge properly "
					+ locRef + " in "
					+ docs.getKey().getName());
			    }
			    successfullyMerged.add(locMerged);
			    break;
			}
		    }
		}
	    } catch (InvocationTargetException e) {
		e.printStackTrace();
		fail("Test case is broken");
	    }
	}
	System.out.println("Total locations in source documents: "+ total_locations);
	System.out.println("Total locations merged: "+total_locations_merged + " - "+(float)total_locations_merged/total_locations*100+"%");
	System.out.println("Merge failed " + fail_count + " times");
	printStats();
	assertEquals("Fail counter isn't zero", 0, fail_count);
    }
    
    private void printStats() {
	MergeProcessor mp = MergeProcessor.getInstance();
	System.out.println("Discarded because parent was null or didn't match: " + mp.parentNull);
	System.out.println("Discarded because common parent wasn't found: " + mp.commonParentNull);
	System.out.println("Discarded because parent was not common: " + mp.parentWrong);
	System.out.println("Discarded bacause span was null: " + mp.spanNull);
	System.out.println("Discarded because text was null: " + mp.textNull);
	System.out.println("Discarded because text and/or span was wrong: " + mp.spanOrTextWrong);
	System.out.println("Discarded because contents were wrong: " + mp.contentWrong);
    }
}
