package com.unitesk.requality.tests.merge.debug;

import java.lang.reflect.InvocationTargetException;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import org.w3c.dom.Document;

import com.unitesk.requality.merge.Fragment;
import com.unitesk.requality.merge.Location;
import com.unitesk.requality.merge.MergeProcessor;
import com.unitesk.requality.merge.Node;
import com.unitesk.requality.merge.Tree;
import com.unitesk.requality.tests.merge.MergeProcessorAccessor;
import com.unitesk.requality.tests.merge.TestDataHelper;
import com.unitesk.requality.tests.merge.Pair;

import junit.framework.TestCase;

public class MergeDebug extends TestCase {

    private MergeProcessor mp;
    private MergeProcessorAccessor mpa;
    private Map<String, Pair<Document, Document>> testData;

    public void setUp() {
	mp = MergeProcessor.getInstance();
	mpa = new MergeProcessorAccessor(mp);
	testData = TestDataHelper.getInstance().getDebugData();
    }

    public void testMergeDebug() {
	Document v1 = testData.get("Debug").getKey();
	Document v2 = testData.get("Debug").getValue();
	assertedMerge(v1, v2);
    }

    public void testMergeWiki() {
	Document v1 = testData.get("Wiki").getKey();
	Document v2 = testData.get("Wiki").getValue();
	assertedMerge(v1, v2);
    }

    public void testMergeRfc() {
	Document v1 = testData.get("Rfc").getKey();
	Document v2 = testData.get("Rfc").getValue();
	assertedMerge(v1, v2);
    }

    public void testMergeImap() {
	Document v1 = testData.get("IMAP").getKey();
	Document v2 = testData.get("IMAP").getValue();
	assertedMerge(v1, v2);
    }
    
    private void assertedMerge(Document v1, Document v2) {
	try {
	    Tree t1 = new Tree(v1);
	    Tree t2 = new Tree(v2);
	    assertTrue("Test data is invalid",
		    t1.getTraversalSequence().size() > t2
			    .getTraversalSequence().size());
	    List<Location> locations = mpa.lookupLocations(t1);
	    assertTrue("No locations were found", locations.size() > 0);
	    locations = mpa.updateLocations(locations);
	    for(Location loc : locations)
		assertNotNull(loc.getCommonParent());
	    locations = mpa.traceLocations(locations, t2);
	    assertTrue("No locations were traced", locations.size() > 0);
	    int nullCounter = 0;
	    for(Location loc : locations)
		if(loc.getAnchorNode()==null)
		    nullCounter++;
	    assertEquals(0, nullCounter);
	    Map<Node, List<Location>> locationMapping = mpa
		    .mapLocations(locations);
	    Iterator<Node> anchorNodes = locationMapping.keySet().iterator();
	    while (anchorNodes.hasNext()) {
		Node anchorNode = anchorNodes.next();
		List<Location> nodeLocations = locationMapping.get(anchorNode);
		for (Location loc : nodeLocations) {
		    assertFalse(loc.getAnchorNode()==null);
		    assertTrue(
			    "Location mapping is invalid",
			    loc.getAnchorNode().getDomNode()
				    .isSameNode(anchorNode.getDomNode()));
		    assertTrue("Location id isn't in merged ids list", mp
			    .getMergedIds().contains(loc.getLocationId()));
		    assertFalse("Location id is in missing ids list", mp
			    .getMissingIds().contains(loc.getLocationId()));
		}
	    }
	    Map<Node, Fragment> fragmentMapping = mpa
		    .makeFragments(locationMapping);
	    assertEquals(
		    "Fragment mapping has different size that location mapping",
		    locationMapping.size(), fragmentMapping.size());
	    Iterator<Node> i = fragmentMapping.keySet().iterator();
	    while (i.hasNext()) {
		int sizeBeforeMerge = t2.getTraversalSequence().size();
		Node anchorNode = i.next();
		mpa.mergeFragment(fragmentMapping.get(anchorNode), anchorNode);
		t2.updateTree();
		assertTrue(t2.getTraversalSequence().size()>sizeBeforeMerge);
	    }
	} catch (InvocationTargetException e) {
	    e.getCause().printStackTrace();
	    fail("Test failed: invoked method has throwed out exception");
	} catch (ClassCastException e) {
	    fail("Broken test case: unexpected type of returned object");
	}
    }
}
