package com.unitesk.requality.tests.merge.sanity;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import com.unitesk.requality.merge.Context;
import com.unitesk.requality.merge.Node;
import com.unitesk.requality.merge.Tree;
import com.unitesk.requality.tests.merge.TestDataHelper;
import com.unitesk.requality.tests.merge.Utils;
import com.unitesk.requality.tests.merge.Pair;

import org.w3c.dom.Document;

import junit.framework.*;

public class ContextTest extends TestCase {

    private Map<String, Pair<Document, Document>> testData;

    public void setUp() {
	testData = TestDataHelper.getInstance().getSanityData();
    }
    
    public void testId() {
	Document doc = testData.get("Sanity").getKey();
	Tree t = new Tree(doc);
	List<Integer> idList = new ArrayList<Integer>();
	for(Node x : t.getTraversalSequence()){
	    idList.add(x.getContext().getId());
	}
	for(int i = 0; i<t.getTraversalSequence().size(); i++){
	    Context current = t.getTraversalSequence().get(i).getContext();
	    assertEquals(i+1, current.getId());
	}
    }
    
    public void testAddChildContext() {
	Document doc = testData.get("Sanity").getKey();
	Tree t = new Tree(doc);
	for (Node x : t.getTraversalSequence()) {
	    Context prevContext = null;
	    for (Context ctx : x.getContext().getChildContexts()) {
		if (prevContext != null) {
		    assertTrue("Context sorting is broken",
			    ctx.getLabel() > prevContext.getLabel());
		    assertTrue("Context sorting is broken",
			    ctx.getId() > prevContext.getId());
		}
		prevContext = ctx;
	    }
	}
    }

    public void testPositionalDistance() {
	Document doc = testData.get("Sanity").getKey();
	Tree t1 = new Tree(doc);
	Tree t2 = new Tree(doc);
	for (Node x : t1.getTraversalSequence()) {
	    Context refContext = x.getContext();
	    int min_distance = Integer.MAX_VALUE;
	    Node nearest = null;
	    for (Node y : t2.getTraversalSequence()) {
		Context currContext = y.getContext();
		int distance = refContext.positionDistance(currContext);
		if (distance < min_distance) {
		    min_distance = distance;
		    nearest = y;
		}
	    }
	    if(!nearest.isParent(x))
		assertTrue(x.getDomNode().isSameNode(nearest.getDomNode()));
	}
    }

    public void testContextSimilarity() {
	Document doc = testData.get("Sanity").getKey();
	Tree t1 = new Tree(doc);
	Tree t2 = new Tree(doc);
	for (Node x : t1.getTraversalSequence()) {
	    if (x.isLeaf())
		continue;
	    Context refContext = x.getContext();
	    int max_score = 0;
	    Node nearest = null;
	    for (Node y : t2.getTraversalSequence()) {
		if (y.isLeaf())
		    continue;
		Context currContext = y.getContext();
		int score = refContext.contextSimilarity(currContext);
		if (score > max_score && x.isEqual(y)) {
		    max_score = score;
		    nearest = y;
		}
	    }
	    assertFalse(nearest == null);
	    assertTrue(x.getDomNode().isSameNode(nearest.getDomNode()));
	}
    }

    public void testIsWithin() {
	Document doc = testData.get("Sanity").getKey();
	Tree t = new Tree(doc);
	Iterator<Node> i = t.traverse();
	Node prev = i.next();
	while (i.hasNext()) {
	    Node curr = i.next();
	    if (Utils.isParent(prev, curr))
		assertTrue(curr.getContext().isWithin(prev.getContext()));
	    else
		assertFalse(curr.getContext().isWithin(prev.getContext()));
	    prev = curr;
	}
    }

    public void testEquals() {
	Document doc = testData.get("Sanity").getKey();
	Tree t1 = new Tree(doc);
	Tree t2 = new Tree(doc);
	Iterator<Node> i = t1.traverse();
	Iterator<Node> j = t2.traverse();
	while (i.hasNext() && j.hasNext()) {
	    Node x = i.next();
	    Node y = j.next();
	    assertFalse(x.getContext().equals(y.getContext()));
	}
    }
}
