package com.unitesk.requality.tests.merge.sanity;

import java.lang.reflect.InvocationTargetException;
import java.util.List;
import java.util.Map;

import junit.framework.TestCase;

import org.w3c.dom.Document;

import com.unitesk.requality.merge.Location;
import com.unitesk.requality.merge.MergeProcessor;
import com.unitesk.requality.merge.Tree;
import com.unitesk.requality.tests.merge.MergeProcessorAccessor;
import com.unitesk.requality.tests.merge.TestDataHelper;
import com.unitesk.requality.tests.merge.Pair;


public class LocationTest extends TestCase {
    
    private Map<String, Pair<Document, Document>> testData;
    
    public void setUp() {
	testData = TestDataHelper.getInstance().getSanityData();
    }
    
    public void testRetrieveLocationText() {
	try {
	    MergeProcessorAccessor mpa = new MergeProcessorAccessor(MergeProcessor.getInstance());
	    Document doc = testData.get("Sanity").getKey();
	    Tree t = new Tree(doc);
	    List<Location> locations = mpa.lookupLocations(t);
	    assertEquals("Test data has changed",4,locations.size());
	    assertTrue(locations.get(0).getAnchorText().equals("requirement text"));
	    assertTrue(locations.get(1).getAnchorText().equals(" text"));
	    assertTrue(locations.get(2).getAnchorText().equals("\n\t\ttext with span in the middle\n\t"));
	    assertTrue(locations.get(3).getAnchorText().equals("bold"));
	} catch (InvocationTargetException e) {
	    e.getCause().printStackTrace();
	    fail("Test failed: invoked method has throwed out exception");
	} catch (ClassCastException e) {
	    fail("Broken test case: unexpected type of returned object");
	}
    }
    
    public void testRetrieveLocationId() {
	try {
	    MergeProcessorAccessor mpa = new MergeProcessorAccessor(MergeProcessor.getInstance());
	    Document doc = testData.get("Sanity").getKey();
	    Tree t = new Tree(doc);
	    List<Location> locations = mpa.lookupLocations(t);
	    assertEquals("Test data has changed",4,locations.size());
	    assertTrue(locations.get(0).getLocationId().equals("1"));
	    assertTrue(locations.get(1).getLocationId().equals("2"));
	    assertTrue(locations.get(2).getLocationId().equals("3"));
	    assertTrue(locations.get(3).getLocationId().equals("2"));
	} catch (InvocationTargetException e) {
	    e.getCause().printStackTrace();
	    fail("Test failed: invoked method has throwed out exception");
	} catch (ClassCastException e) {
	    fail("Broken test case: unexpected type of returned object");
	}
    }
}
